@@ -6,7 +6,7 @@
  *                                                                          *
  *                              C Header File                               *
  *                                                                          *
- *       Copyright (C) 1992-2005, 2007 Free Software Foundation, Inc.       *
+ *          Copyright (C) 1992-2007, Free Software Foundation, Inc.         *
  *                                                                          *
  * GNAT is free software;  you can  redistribute it  and/or modify it under *
  * terms of the  GNU General Public License as published  by the Free Soft- *
@@ -14,9 +14,9 @@
  * sion.  GNAT is distributed in the hope that it will be useful, but WITH- *
  * OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY *
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
- * for  more details.  You should have received a copy of the GNU General   *
- * Public License along with GCC; see the file COPYING3.  If not see        *
- * <http://www.gnu.org/licenses/>.                                          *
+ * for  more details.  You should have  received  a copy of the GNU General *
+ * Public License  distributed with GNAT; see file COPYING3.  If not, go to *
+ * http://www.gnu.org/licenses for a complete copy of the license.          *
  *                                                                          *
  * GNAT was originally developed  by the GNAT team at  New York University. *
  * Extensive contributions were provided by Ada Core Technologies Inc.      *
@@ -327,6 +327,10 @@ typedef Int Mechanism_Type;
 #define By_Descriptor_SB   (-8)
 #define By_Descriptor_A    (-9)
 #define By_Descriptor_NCA  (-10)
+#define By_Descriptor_Last (-10)
+
+/* Internal to Gigi.  */
+#define By_Copy_Return     (-128)
 
 /* Definitions of Reason codes for Raise_xxx_Error nodes */
 #define CE_Access_Check_Failed              0
@@ -347,22 +351,22 @@ typedef Int Mechanism_Type;
 #define PE_Access_Before_Elaboration       14
 #define PE_Accessibility_Check_Failed      15
 #define PE_All_Guards_Closed               16
-#define PE_Duplicated_Entry_Address        17
-#define PE_Explicit_Raise                  18
-#define PE_Finalize_Raised_Exception       19
-#define PE_Implicit_Return                 20
-#define PE_Misaligned_Address_Value        21
-#define PE_Missing_Return                  22
-#define PE_Overlaid_Controlled_Object      23
-#define PE_Potentially_Blocking_Operation  24
-#define PE_Stubbed_Subprogram_Called       25
-#define PE_Unchecked_Union_Restriction     26
-#define PE_Illegal_RACW_E_4_18             27
+#define PE_Current_Task_In_Entry_Body      17
+#define PE_Duplicated_Entry_Address        18
+#define PE_Explicit_Raise                  19
+#define PE_Finalize_Raised_Exception       20
+#define PE_Implicit_Return                 21
+#define PE_Misaligned_Address_Value        22
+#define PE_Missing_Return                  23
+#define PE_Overlaid_Controlled_Object      24
+#define PE_Potentially_Blocking_Operation  25
+#define PE_Stubbed_Subprogram_Called       26
+#define PE_Unchecked_Union_Restriction     27
+#define PE_Non_Transportable_Actual        28
 
-#define SE_Empty_Storage_Pool              28
-#define SE_Explicit_Raise                  29
-#define SE_Infinite_Recursion              30
-#define SE_Object_Too_Large                31
-#define SE_Restriction_Violation           32
+#define SE_Empty_Storage_Pool              29
+#define SE_Explicit_Raise                  30
+#define SE_Infinite_Recursion              31
+#define SE_Object_Too_Large                32
 
-#define LAST_REASON_CODE                   31
+#define LAST_REASON_CODE                   32
