@@ -1,8 +1,9 @@
 /* PR target/17565.  GCC used to put the asm into the delay slot
    of the call.  */
 /* { dg-do assemble } */
-/* { dg-mips-options "-O -mno-mips16" } */
-int foo (int n)
+/* { dg-mips-options "-O" } */
+
+NOMIPS16 int foo (int n)
 {
   register int k asm ("$16") = n;
   if (k > 0)
