@@ -1,6 +1,6 @@
 // Low-level functions for atomic operations: m68k version -*- C++ -*-
 
-// Copyright (C) 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+// Copyright (C) 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -27,10 +27,10 @@
 // invalidate any other reasons why the executable file might be covered by
 // the GNU General Public License.
 
-#include <bits/atomicity.h>
+#include <ext/atomicity.h>
+
+_GLIBCXX_BEGIN_NAMESPACE(__gnu_cxx)
 
-namespace __gnu_cxx
-{
 #if ( defined(__mc68020__) || defined(__mc68030__) \
       || defined(__mc68040__) || defined(__mc68060__) ) \
     && !defined(__mcpu32__)
@@ -89,14 +89,14 @@ namespace __gnu_cxx
     _Atomic_word __result;
     
     // bset with no immediate addressing (not SMP-safe)
-#if defined(__mcf5200__) || defined(__mcf5300__)
+#if defined(__mcfisaa__) || defined(__mcfisaaplus__)
     __asm__ __volatile__("1: bset.b #7,%0@\n\tjbne 1b"
 			 : /* no outputs */
 			 : "a"(&_Atomicity_lock<0>::_S_atomicity_lock)
 			 : "cc", "memory");
     
-    // CPU32 and MCF5400 support test-and-set (SMP-safe).
-#elif defined(__mcpu32__) || defined(__mcf5400__)
+    // CPU32 and CF ISAs B & C support test-and-set (SMP-safe).
+#elif defined(__mcpu32__) || defined(__mcfisab__) || defined (__mcfisac__)
     __asm__ __volatile__("1: tas %0\n\tjbne 1b"
 			 : "+m"(_Atomicity_lock<0>::_S_atomicity_lock)
 			 : /* none */
@@ -130,4 +130,5 @@ namespace __gnu_cxx
     // architecturally guaranteed to be atomic.
     __exchange_and_add(__mem, __val);
   }
-} // namespace __gnu_cxx
+
+_GLIBCXX_END_NAMESPACE
