@@ -1,6 +1,7 @@
 // std::time_get, std::time_put implementation, GNU version -*- C++ -*-
 
-// Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
+// Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -15,7 +16,7 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 // As a special exception, you may use this file as part of a free software
@@ -27,6 +28,11 @@
 // invalidate any other reasons why the executable file might be covered by
 // the GNU General Public License.
 
+/** @file time_members.h
+ *  This is an internal header file, included by other library headers.
+ *  You should not attempt to use it directly.
+ */
+
 //
 // ISO C++ 14882: 22.2.5.1.2 - time_get functions
 // ISO C++ 14882: 22.2.5.3.2 - time_put functions
@@ -34,47 +40,53 @@
 
 // Written by Benjamin Kosnik <bkoz@redhat.com>
 
+_GLIBCXX_BEGIN_NAMESPACE(std)
+
   template<typename _CharT>
     __timepunct<_CharT>::__timepunct(size_t __refs) 
-    : facet(__refs), _M_data(NULL)
-    { 
-#if !(__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 2))
-      _M_name_timepunct = _S_get_c_name();
-#endif
-      _M_initialize_timepunct(); 
-    }
+    : facet(__refs), _M_data(NULL), _M_c_locale_timepunct(NULL), 
+      _M_name_timepunct(_S_get_c_name())
+    { _M_initialize_timepunct(); }
 
   template<typename _CharT>
     __timepunct<_CharT>::__timepunct(__cache_type* __cache, size_t __refs) 
-    : facet(__refs), _M_data(__cache)
-    { 
-#if !(__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 2))
-      _M_name_timepunct = _S_get_c_name();
-#endif
-      _M_initialize_timepunct(); 
-    }
+    : facet(__refs), _M_data(__cache), _M_c_locale_timepunct(NULL), 
+      _M_name_timepunct(_S_get_c_name())
+    { _M_initialize_timepunct(); }
 
   template<typename _CharT>
-    __timepunct<_CharT>::__timepunct(__c_locale __cloc, 
-				 const char* __s __attribute__ ((__unused__)), 
+    __timepunct<_CharT>::__timepunct(__c_locale __cloc, const char* __s,
 				     size_t __refs) 
-    : facet(__refs), _M_data(NULL)
-    { 
-#if !(__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 2))
-      char* __tmp = new char[std::strlen(__s) + 1];
-      std::strcpy(__tmp, __s);
-      _M_name_timepunct = __tmp;
-#endif
-      _M_initialize_timepunct(__cloc); 
+    : facet(__refs), _M_data(NULL), _M_c_locale_timepunct(NULL), 
+      _M_name_timepunct(NULL)
+    {
+      if (__builtin_strcmp(__s, _S_get_c_name()) != 0)
+	{
+	  const size_t __len = __builtin_strlen(__s) + 1;
+	  char* __tmp = new char[__len];
+	  __builtin_memcpy(__tmp, __s, __len);
+	  _M_name_timepunct = __tmp;
+	}
+      else
+	_M_name_timepunct = _S_get_c_name();
+
+      try
+	{ _M_initialize_timepunct(__cloc); }
+      catch(...)
+	{
+	  if (_M_name_timepunct != _S_get_c_name())
+	    delete [] _M_name_timepunct;
+	  __throw_exception_again;
+	}
     }
 
   template<typename _CharT>
     __timepunct<_CharT>::~__timepunct()
     { 
-#if !(__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 2))
       if (_M_name_timepunct != _S_get_c_name())
 	delete [] _M_name_timepunct;
-#endif
       delete _M_data; 
       _S_destroy_c_locale(_M_c_locale_timepunct); 
     }
+
+_GLIBCXX_END_NAMESPACE
