@@ -47,12 +47,12 @@ protected:
   {
     unshift_called = true;
     to_next = to;
-    return codecvt_base::error;
+    return std::codecvt_base::ok;
   }
 };
 
 // libstdc++/12790
-// basic_filebuf::close() should fail if codecvt::unshift() fails
+// basic_filebuf::close() should call codecvt::unshift()
 void test01()
 {
   using namespace std;
@@ -68,11 +68,11 @@ void test01()
 
   fb.open(name, ios_base::out);
   fb.sputc('a');
+  fb.pubsync(); // Does not call unshift()
 
   VERIFY( !cvt->unshift_called );
-  filebuf* ret = fb.close();
+  fb.close();
   VERIFY( cvt->unshift_called );
-  VERIFY( ret == NULL );
 }
 
 int main()
