@@ -18,6 +18,8 @@
 
 // 27.8.1.4 Overridden virtual functions
 
+// { dg-require-fileio "" }
+
 #include <locale>
 #include <fstream>
 #include <testsuite_hooks.h>
@@ -50,8 +52,7 @@ protected:
 };
 
 // libstdc++/12790
-// basic_filebuf::seekoff() should call codecvt::unshift(),
-// but only if writing
+// basic_filebuf::seekoff() should call codecvt::unshift()
 void test01()
 {
   using namespace std;
@@ -66,10 +67,12 @@ void test01()
   fb.pubimbue(loc);
 
   fb.open(name, ios_base::out);
+  fb.sputc('a');
+  fb.pubsync(); // Does not call unshift()
 
   VERIFY( !cvt->unshift_called );
   fb.pubseekoff(0, ios_base::cur);
-  VERIFY( !cvt->unshift_called );
+  VERIFY( cvt->unshift_called );
 }
 
 int main()
