@@ -50,7 +50,8 @@ protected:
 };
 
 // libstdc++/12790
-// basic_filebuf::seekoff() should call codecvt::unshift()
+// basic_filebuf::seekoff() should call codecvt::unshift(),
+// but only if writing
 void test01()
 {
   using namespace std;
@@ -65,12 +66,10 @@ void test01()
   fb.pubimbue(loc);
 
   fb.open(name, ios_base::out);
-  fb.sputc(L'a');
-  fb.pubsync(); // Does not call unshift()
 
   VERIFY( !cvt->unshift_called );
   fb.pubseekoff(0, ios_base::cur);
-  VERIFY( cvt->unshift_called );
+  VERIFY( !cvt->unshift_called );
 }
 
 int main()
