@@ -24,28 +24,40 @@
 #include <locale>
 #include <testsuite_hooks.h>
 
-class Buf1 : public std::wstreambuf
+class Buf2 : public std::wstreambuf
 {
+public:
+  std::locale before;
+  std::locale after;
+
 protected:
-  void imbue(const std::locale&)
-  { }
+  void imbue(const std::locale& loc)
+  {
+    before = getloc();
+
+    std::wstreambuf::imbue(loc);
+
+    after = getloc();
+  }
 };
 
 // libstdc++/13007
-void test01()
+void test02()
 {
   bool test __attribute__((unused)) = true;
-  
-  Buf1 buf;
-  std::locale loc(std::locale("is_IS.UTF-8"));
+
+  Buf2 buf;
+  std::locale loc(std::locale("en_US"));
 
   buf.pubimbue(loc);
 
   VERIFY( buf.getloc() == loc );
+  VERIFY( buf.before == std::locale::classic() );
+  VERIFY( buf.after == std::locale::classic() );
 }
 
 int main()
 {
-  test01();
+  test02();
   return 0;
 }
