@@ -23,26 +23,17 @@
 #include <iterator>
 #include <testsuite_hooks.h>
 
-class Fred
-{
-public:
-  Fred(bool)
-  { }
-};
-
-void barney(Fred)
-{ }
-
 // libstdc++/14320
 void test01()
 {
   using namespace std;
   bool test __attribute__((unused)) = true;
 
-  typedef istreambuf_iterator<char>::difference_type Distance;
-  
-  Distance d = 0;
-  barney(d);
+  typedef std::istreambuf_iterator<char>::difference_type Distance;
+  Distance d = 2;
+  Distance e = 3;
+  d *= e;
+  VERIFY( static_cast<int>(d) == 6 );
 }
 
 int main()
