@@ -1,3 +1,6 @@
+// Derived from libstdc++/12048 by LJR <ljrittle@acm.org> with
+// reminder from Petur Runolfsson <peturr02@ru.is>.
+
 // Copyright (C) 2003 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
@@ -16,8 +19,14 @@
 // Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
+// DR 49 states that cin.rdbuf()->sbumpc() and fgetc(stdin) should be
+// equivalent and interchangable. Currently however, cin.rdbuf()->sungetc()
+// only returns characters that were read with cin.rdbuf()->sbumpc()
+
 // { dg-require-fileio "" }
 
+// { dg-do run { xfail *-*-* } }
+
 #include <iostream>
 #include <cstdio>
 #include <testsuite_hooks.h>
@@ -29,11 +38,16 @@ test01()
 
   char c1;
   int c2;
+  char c3;
   std::cin.get(c1);
-  std::cin.unget();
-  VERIFY( std::cin.good() );
   c2 = std::fgetc(stdin);
-  VERIFY( c2 == std::char_traits<char>::to_int_type(c1) );
+  std::cin.unget();
+  if (std::cin.good())
+    {
+      std::cin.get(c3);
+      VERIFY( std::cin.good() );
+      VERIFY( c3 == std::char_traits<char>::to_char_type(c2) );
+    }
 }
 
 int main(void)
